/****************************************************************************
 **             - iom16.h -
 **
 **     This file declares the internal register addresses for ATmega16.
 **
 **     Used with iccAVR and aAVR.
 **
 **     Copyright IAR Systems 2000. All rights reserved.
 **
 **     File version: $Revision: 1.6 $
 **
 ***************************************************************************/

#include "iomacro.h"

#if TID_GUARD(3)
#error This file should only be compiled with iccavr or aavr whith processor option -v3
#endif /* TID_GUARD(3) */

#ifdef __HAS_RAMPZ__
#error This file should not have RAMPZ enabled, use --cpu or --64k_flash
#endif /* __HAS_RAMPZ__ */

/* Include the SFR part if this file has not been included before,
 * OR this file is included by the assembler (SFRs must be defined in
 * each assembler module). */
#if !defined(__IOM16_H) || defined(__IAR_SYSTEMS_ASM__)

#pragma language=extended

/*==========================*/
/* Predefined SFR Addresses */
/*==========================*/

/****************************************************************************
 * An example showing the SFR_B() macro call, 
 * the expanded result and usage of this result:
 *
 * SFR_B(AVR,   0x1F) Expands to:
 * __io union {
 *             unsigned char AVR;                 // The sfrb as 1 byte
 *             struct {                           // The sfrb as 8 bits
 *                     unsigned char AVR_Bit0:1,
 *                                   AVR_Bit1:1,
 *                                   AVR_Bit2:1,
 *                                   AVR_Bit3:1,
 *                                   AVR_Bit4:1,
 *                                   AVR_Bit5:1,
 *                                   AVR_Bit6:1,
 *                                   AVR_Bit7:1;
 *                    };
 *            } @ 0x1F;
 * Examples of how to use the expanded result:
 * AVR |= (1<<5);
 * or like this:
 * AVR_Bit5 = 1;
 ***************************************************************************/

SFR_B_N(0x3F,SREG,I,T,H,S,V,N,Z,C)
SFR_W_N(0x3D,SP,Dummy15,Dummy14,Dummy13,Dummy12,Dummy11,SP10,SP9,SP8,SP7,SP6,SP5,SP4,SP3,SP2,SP1,SP0)
SFR_B_R(0x3C,OCR0)
SFR_B_N(0x3B,GICR,INT1,INT0,INT2,Dummy4,Dummy3,Dummy2,IVSEL,IVCE)
SFR_B_N(0x3A,GIFR,INTF1,INTF0,INTF2,Dummy4,Dummy3,Dummy2,Dummy1,Dummy0)
SFR_B_N(0x39,TIMSK,OCIE2,TOIE2,TICIE1,OCIE1A,OCIE1B,TOIE1,OCIE0,TOIE0)
SFR_B_N(0x38,TIFR,OCF2,TOV2,ICF1,OCF1A,OCF1B,TOV1,OCF0,TOV0)
SFR_B_N(0x37,SPMCR,SPMIE,RWWSB,Dummy5,RWWSRE,BLBSET,PGWRT,PGERS,SPMEN)
SFR_B_N(0x36,TWCR,TWINT,TWEA,TWSTA,TWSTO,TWWC,TWEN,Dummy1,TWIE)
SFR_B_N(0x35,MCUCR,SM2,SE,SM1,SM0,ISC11,ISC10,ISC01,ISC00)
SFR_B_N(0x34,MCUCSR,JTD,ISC2,Dummy5,JTRF,WDRF,BORF,EXTRF,PORF)
SFR_B_N(0x33,TCCR0,FOC0,WGM00,COM01,COM00,WGM01,CS02,CS01,CS00)
SFR_B_R(0x32,TCNT0)
SFR_B2_R(0x31,OSCCAL,OCDR)
SFR_B_N(0x30,SFIOR,ADTS2,ADTS1,ADTS0,Dummy4,ACME,PUD,PSR2,PSR10)
SFR_B_N(0x2F,TCCR1A,COM1A1,COM1A0,COM1B1,COM1B0,FOC1A,FOC1B,WGM11,WGM10)
SFR_B_N(0x2E,TCCR1B,ICNC1,ICES1,Dummy5,WGM13,WGM12,CS12,CS11,CS10)
SFR_W_R(0x2C,TCNT1)
SFR_W_R(0x2A,OCR1A)
SFR_W_R(0x28,OCR1B)
SFR_W_R(0x26,ICR1)
SFR_B_N(0x25,TCCR2,FOC2,WGM20,COM21,COM20,WGM21,CS22,CS21,CS20)
SFR_B_R(0x24,TCNT2)
SFR_B_R(0x23,OCR2)
SFR_B_N(0x22,ASSR,Dummy7,Dummy6,Dummy5,Dummy4,AS2,TCN2UB,OCR2UB,TCR2UB)
SFR_B_N(0x21,WDTCR,Dummy7,Dummy6,Dummy5,WDTOE,WDE,WDP2,WDP1,WDP0)
SFR_B2_N(0x20,UBRRH,UCSRC,URSEL,UMSEL,UPM1,UPM0,USBS,UCSZ1,UCSZ0,UCPOL)
SFR_W_R(0x1E,EEAR)
SFR_B_R(0x1D,EEDR)
SFR_B_N(0x1C,EECR,Dummy7,Dummy6,Dummy5,Dummy4,EERIE,EEMWE,EEWE,EERE)
SFR_B_N(0x1B,PORTA,PORTA7,PORTA6,PORTA5,PORTA4,PORTA3,PORTA2,PORTA1,PORTA0)
SFR_B_N(0x1A,DDRA,DDA7,DDA6,DDA5,DDA4,DDA3,DDA2,DDA1,DDA0)
SFR_B_N(0x19,PINA,PINA7,PINA6,PINA5,PINA4,PINA3,PINA2,PINA1,PINA0)
SFR_B_N(0x18,PORTB,PORTB7,PORTB6,PORTB5,PORTB4,PORTB3,PORTB2,PORTB1,PORTB0)
SFR_B_N(0x17,DDRB,DDB7,DDB6,DDB5,DDB4,DDB3,DDB2,DDB1,DDB0)
SFR_B_N(0x16,PINB,PINB7,PINB6,PINB5,PINB4,PINB3,PINB2,PINB1,PINB0)
SFR_B_N(0x15,PORTC,PORTC7,PORTC6,PORTC5,PORTC4,PORTC3,PORTC2,PORTC1,PORTC0)
SFR_B_N(0x14,DDRC,DDC7,DDC6,DDC5,DDC4,DDC3,DDC2,DDC1,DDC0)
SFR_B_N(0x13,PINC,PINC7,PINC6,PINC5,PINC4,PINC3,PINC2,PINC1,PINC0)
SFR_B_N(0x12,PORTD,PORTD7,PORTD6,PORTD5,PORTD4,PORTD3,PORTD2,PORTD1,PORTD0)
SFR_B_N(0x11,DDRD,DDD7,DDD6,DDD5,DDD4,DDD3,DDD2,DDD1,DDD0)
SFR_B_N(0x10,PIND,PIND7,PIND6,PIND5,PIND4,PIND3,PIND2,PIND1,PIND0)
SFR_B_R(0x0F,SPDR)
SFR_B_N(0x0E,SPSR,SPIF,WCOL,Dummy5,Dummy4,Dummy3,Dummy2,Dummy1,SPI2X)
SFR_B_N(0x0D,SPCR,SPIE,SPE,DORD,MSTR,CPOL,CPHA,SPR1,SPR0)
SFR_B_R(0x0C,UDR)
SFR_B_N(0x0B,UCSRA,RXC,TXC,UDRE,FE,DOR,PE,U2X,MPCM)
SFR_B_N(0x0A,UCSRB,RXCIE,TXCIE,UDRIE,RXEN,TXEN,UCSZ2,RXB8,TXB8)
SFR_B_R(0x09,UBRRL)
SFR_B_N(0x08,ACSR,ACD,ACBG,ACO,ACI,ACIE,ACIC,ACIS1,ACIS0)
SFR_B_N(0x07,ADMUX,REFS1,REFS0,ADLAR,MUX4,MUX3,MUX2,MUX1,MUX0)
SFR_B_N(0x06,ADCSRA,ADEN,ADSC,ADATE,ADIF,ADIE,ADPS2,ADPS1,ADPS0)
SFR_W_R(0x04,ADC)
SFR_B_R(0x03,TWDR)
SFR_B_N(0x02,TWAR,TWA6,TWA5,TWA4,TWA3,TWA2,TWA1,TWA0,TWGCE)
SFR_B_N(0x01,TWSR,TWS7,TWS6,TWS5,TWS4,TWS3,Dummy2,TWPS1,TWPS0)
SFR_B_R(0x00,TWBR)

#ifndef __IOM16_H
#define __IOM16_H

/* SFRs are local in assembler modules (so this file may need to be */
/* included in more than one module in the same source file), */
/* but #defines must only be made once per source file. */

/*==============================*/
/* Interrupt Vector Definitions */
/*==============================*/

/* NB! vectors are specified as byte addresses */

#define    RESET_vect           (0x00)
#define    INT0_vect            (0x04)
#define    INT1_vect            (0x08)
#define    TIMER2_COMP_vect     (0x0C)
#define    TIMER2_OVF_vect      (0x10)
#define    TIMER1_CAPT_vect     (0x14)
#define    TIMER1_COMPA_vect    (0x18)
#define    TIMER1_COMPB_vect    (0x1C)
#define    TIMER1_OVF_vect      (0x20)
#define    TIMER0_OVF_vect      (0x24)
#define    SPI_STC_vect         (0x28)
#define    USART_RXC_vect       (0x2C)
#define    USART_UDRE_vect      (0x30)
#define    USART_TXC_vect       (0x34)
#define    ADC_vect             (0x38)
#define    EE_RDY_vect          (0x3C)
#define    ANA_COMP_vect        (0x40)
#define    TWI_vect             (0x44)
#define    INT2_vect            (0x48)
#define    TIMER0_COMP_vect     (0x4C)
#define    SPM_RDY_vect         (0x50)

#ifdef __IAR_SYSTEMS_ASM__   
#ifndef ENABLE_BIT_DEFINITIONS
#define  ENABLE_BIT_DEFINITIONS
#endif /* ENABLE_BIT_DEFINITIONS */
#endif /* __IAR_SYSTEMS_ASM__ */

#ifdef ENABLE_BIT_DEFINITIONS

/* Bit definitions for use with the IAR Assembler   
   The Register Bit names are represented by their bit number (0-7).
*/
 
/* General Interrupt Control register */
#define    INT1     7
#define    INT0     6
#define    INT2     5

#define    IVSEL    1
#define    IVCE     0
 
/* General Interrupt Flag Register */
#define    INTF1    7
#define    INTF0    6
#define    INTF2    5
 
/* Timer/Counter Interrupt MaSK register */
#define    OCIE2    7
#define    TOIE2    6
#define    TICIE1   5
#define    OCIE1A   4
#define    OCIE1B   3
#define    TOIE1    2
#define    OCIE0    1
#define    TOIE0    0
 
/* Timer/Counter Interrupt Flag register */
#define    OCF2     7
#define    TOV2     6
#define    ICF1     5
#define    OCF1A    4
#define    OCF1B    3
#define    TOV1     2
#define    OCF0     1
#define    TOV0     0

/* SPM Control Register */			
#define    SPMIE    7
#define    RWWSB    6
#define    RWWSRE   4
#define    BLBSET   3
#define    PGWRT    2
#define    PGERS    1
#define    SPMEN    0

/* TWI Control Register */
#define    TWINT    7
#define    TWEA     6
#define    TWSTA    5
#define    TWSTO    4
#define    TWWC     3
#define    TWEN     2
#define    TWIE     0

/* MCU general Control Register */
#define    SM2      7
#define    SE       6
#define    SM1      5
#define    SM0      4
#define    ISC11    3
#define    ISC10    2
#define    ISC01    1
#define    ISC00    0

/* MCU general Control and Status Register */
#define    JTD      7
#define    ISC2     6
#define    JTRF     4
#define    WDRF     3
#define    BORF     2
#define    EXTRF    1
#define    PORF     0
 
/* Timer/Counter 0 Control Register */
#define    FOC0     7
#define    WGM00    6
#define    COM01    5
#define    COM00    4
#define    WGM01    3
#define    CS02     2
#define    CS01     1
#define    CS00     0
  
/* Special Function I/O Register */		
#define    ADTS2    7
#define    ADTS1    6
#define    ADTS0    5
#define    ADHSN    4
#define    ACME     3
#define    PUD      2
#define    PSR2     1
#define    PSR10    0

/* Timer/Counter 1A Control Register */
#define    COM1A1   7
#define    COM1A0   6
#define    COM1B1   5
#define    COM1B0   4
#define    FOC1A    3
#define    FOC1B    2
#define    WGM11    1
#define    WGM10    0
 
/* Timer/Counter 1B Control Register */
#define    ICNC1    7
#define    ICES1    6
#define    WGM13    4
#define    WGM12    3
#define    CS12     2
#define    CS11     1
#define    CS10     0
 
/* Timer/Counter 2 Control Register */
#define    FOC2     7
#define    WGM20    6
#define    COM21    5
#define    COM20    4
#define    WGM21    3
#define    CS22     2
#define    CS21     1
#define    CS20     0
 
/* Asynchronous mode Status Register */
#define    AS2      3
#define    TCN2UB   2
#define    OCR2UB   1
#define    TCR2UB   0
                        
/* Watchdog Timer Control Register */
#define    WDTOE    4
#define    WDE      3
#define    WDP2     2
#define    WDP1     1
#define    WDP0     0
 
/* UART UBRRH and UCSRC */
#define    URSEL    7
#define    UMSEL    6
#define    UPM1     5
#define    UPM0     4
#define    USBS     3
#define    UCSZ1    2
#define    UCSZ0    1
#define    UCPOL    0

/* EEPROM Control Register */
#define    EERIE    3
#define    EEMWE    2
#define    EEWE     1
#define    EERE     0
 
/* Data Register, Port A */
#define    PA7      7
#define    PA6      6
#define    PA5      5
#define    PA4      4
#define    PA3      3
#define    PA2      2
#define    PA1      1
#define    PA0      0
 
/* Data Register, Port A */
#define    PORTA7   7
#define    PORTA6   6
#define    PORTA5   5
#define    PORTA4   4
#define    PORTA3   3
#define    PORTA2   2
#define    PORTA1   1
#define    PORTA0   0
 
/* Data Direction Register, Port A */
#define    DDA7     7
#define    DDA6     6
#define    DDA5     5
#define    DDA4     4
#define    DDA3     3
#define    DDA2     2
#define    DDA1     1
#define    DDA0     0
 
/* Input Pins, Port A */
#define    PINA7    7
#define    PINA6    6
#define    PINA5    5
#define    PINA4    4
#define    PINA3    3
#define    PINA2    2
#define    PINA1    1
#define    PINA0    0
 
/* Data Register, Port B */
#define    PB7      7
#define    PB6      6
#define    PB5      5
#define    PB4      4
#define    PB3      3
#define    PB2      2
#define    PB1      1
#define    PB0      0
 
 /* Data Register, Port B */
#define    PORTB7   7
#define    PORTB6   6
#define    PORTB5   5
#define    PORTB4   4
#define    PORTB3   3
#define    PORTB2   2
#define    PORTB1   1
#define    PORTB0   0
 
/* Data Direction Register, Port B */
#define    DDB7     7
#define    DDB6     6
#define    DDB5     5
#define    DDB4     4
#define    DDB3     3
#define    DDB2     2
#define    DDB1     1
#define    DDB0     0
 
/* Input Pins, Port B */
#define    PINB7    7
#define    PINB6    6
#define    PINB5    5
#define    PINB4    4
#define    PINB3    3
#define    PINB2    2
#define    PINB1    1
#define    PINB0    0
 
/* Data Register, Port C */
#define    PC7      7
#define    PC6      6
#define    PC5      5
#define    PC4      4
#define    PC3      3
#define    PC2      2
#define    PC1      1
#define    PC0      0
 
  /* Data Register, Port C */
#define    PORTC7   7
#define    PORTC6   6
#define    PORTC5   5
#define    PORTC4   4
#define    PORTC3   3
#define    PORTC2   2
#define    PORTC1   1
#define    PORTC0   0
 
/* Data Direction Register, Port C */
#define    DDC7     7
#define    DDC6     6
#define    DDC5     5
#define    DDC4     4
#define    DDC3     3
#define    DDC2     2
#define    DDC1     1
#define    DDC0     0
 
/* Input Pins, Port C */
#define    PINC7    7
#define    PINC6    6
#define    PINC5    5
#define    PINC4    4
#define    PINC3    3
#define    PINC2    2
#define    PINC1    1
#define    PINC0    0
 
/* Data Register, Port D */
#define    PD7      7
#define    PD6      6
#define    PD5      5
#define    PD4      4
#define    PD3      3
#define    PD2      2
#define    PD1      1
#define    PD0      0
 
  /* Data Register, Port D */
#define    PORTD7   7
#define    PORTD6   6
#define    PORTD5   5
#define    PORTD4   4
#define    PORTD3   3
#define    PORTD2   2
#define    PORTD1   1
#define    PORTD0   0
 
/* Data Direction Register, Port D */
#define    DDD7     7
#define    DDD6     6
#define    DDD5     5
#define    DDD4     4
#define    DDD3     3
#define    DDD2     2
#define    DDD1     1
#define    DDD0     0
 
/* Input Pins, Port D */
#define    PIND7    7
#define    PIND6    6
#define    PIND5    5
#define    PIND4    4
#define    PIND3    3
#define    PIND2    2
#define    PIND1    1
#define    PIND0    0
  
/* SPI Status Register */
#define    SPIF     7
#define    WCOL     6
#define    SPI2X    0

/* SPI Control Register */
#define    SPIE     7
#define    SPE      6
#define    DORD     5
#define    MSTR     4
#define    CPOL     3
#define    CPHA     2
#define    SPR1     1
#define    SPR0     0

/* UART Control and Status Register A */
#define    RXC      7
#define    TXC      6
#define    UDRE     5
#define    FE       4
#define    DOR      3
#define    PE       2
#define    U2X      1
#define    MPCM     0
 
/* UART Control and Status Register B */
#define    RXCIE    7
#define    TXCIE    6
#define    UDRIE    5
#define    RXEN     4
#define    TXEN     3
#define    UCSZ2    2
#define    RXB8     1
#define    TXB8     0
 
/* Analog Comparator Control and Status Register */
#define    ACD      7
#define    ACBG     6
#define    ACO      5
#define    ACI      4
#define    ACIE     3
#define    ACIC     2
#define    ACIS1    1
#define    ACIS0    0
 
/* ADC MUX */
#define    REFS1    7
#define    REFS0    6
#define    ADLAR    5
#define    MUX4     4
#define    MUX3     3
#define    MUX2     2
#define    MUX1     1
#define    MUX0     0
 
/* ADC Control and Status Register */
#define    ADEN     7
#define    ADSC     6
#define    ADATE    5
#define    ADIF     4
#define    ADIE     3
#define    ADPS2    2
#define    ADPS1    1
#define    ADPS0    0
 
/* TWI Address Register */

#define    TWA6     7
#define    TWA5     6
#define    TWA4     5
#define    TWA3     4
#define    TWA2     3
#define    TWA1     2
#define    TWA0     1
#define    TWGCE    0

/* TWI status register */
#define    TWS7     7
#define    TWS6     6
#define    TWS5     5
#define    TWS4     4
#define    TWS3     3
#define    TWPS1    1
#define    TWPS0    0
 
/* Pointer definition   */
#define    XL     r26
#define    XH     r27
#define    YL     r28
#define    YH     r29
#define    ZL     r30
#define    ZH     r31
 
/* Constants */
#define    RAMEND   0x045F    /* Last On-Chip SRAM location */
#define    XRAMEND  0x045F
#define    E2END    0x01FF
#define    FLASHEND 0x3FFF
 
#endif /* ENABLE_BIT_DEFINITIONS */ 
#endif /* __IOM16_H (define part) */

#pragma language=default

#endif /* __IOM16_H (SFR part) */
